@@ -1,4 +1,4 @@
-#' plot a fitted scaling model
+#' Plot a fitted scaling model
 #' 
 #' Plot the results of a fitted scaling model, from (e.g.) a predicted 
 #' \link{textmodel_wordscores} model or a fitted \link{textmodel_wordfish} 
@@ -31,7 +31,7 @@
 #' @export
 #' @author Kenneth Benoit, Stefan Müller, and Adam Obeng
 #' @seealso \code{\link{textmodel_wordfish}}, \code{\link{textmodel_wordscores}}, 
-#'   \code{\link{coef.textmodel}}
+#'   \code{\link{textmodel_ca}}
 #' @keywords textplot
 #' @examples
 #' \dontrun{
@@ -41,65 +41,78 @@
 #' 
 #' ## wordscores
 #' refscores <- c(rep(NA, 4), -1, 1, rep(NA, 8))
-#' ws <- textmodel(ie_dfm, refscores, model="wordscores", smooth = 1)
-#' pred <- predict(ws)
+#' ws <- textmodel_wordscores(ie_dfm, refscores, smooth = 1)
 #' # plot estimated word positions
-#' textplot_scale1d(pred, margin = "features", 
-#'                  highlighted = c("minister", "have", "our", "budget"))
+#' textplot_scale1d(ws, highlighted = c("minister", "have", "our", "budget"))
 #' # plot estimated document positions
-#' textplot_scale1d(pred, margin = "documents",
-#'                  doclabels = doclab,
+#' textplot_scale1d(predict(ws), doclabels = doclab,
 #'                  groups = docvars(data_corpus_irishbudget2010, "party"))
 #'
 #' ## wordfish
-#' wfm <- textmodel_wordfish(dfm(data_corpus_irishbudget2010), dir = c(6,5))
+#' wf <- textmodel_wordfish(dfm(data_corpus_irishbudget2010), dir = c(6,5))
 #' # plot estimated document positions
-#' textplot_scale1d(wfm, doclabels = doclab)
-#' textplot_scale1d(wfm, doclabels = doclab,
+#' textplot_scale1d(wf, doclabels = doclab)
+#' textplot_scale1d(wf, doclabels = doclab,
 #'                  groups = docvars(data_corpus_irishbudget2010, "party"))
 #' # plot estimated word positions
-#' textplot_scale1d(wfm, margin = "features", 
+#' textplot_scale1d(wf, margin = "features", 
 #'                  highlighted = c("government", "global", "children", 
 #'                                  "bank", "economy", "the", "citizenship",
 #'                                  "productivity", "deficit"))
 #'
 #' ## correspondence analysis
-#' wca <- textmodel_ca(ie_dfm)
+#' ca <- textmodel_ca(ie_dfm)
 #' # plot estimated document positions
-#' textplot_scale1d(wca, margin = "documents",
+#' textplot_scale1d(ca, margin = "documents",
 #'                  doclabels = doclab,
 #'                  groups = docvars(data_corpus_irishbudget2010, "party"))
 #' }
-textplot_scale1d <- function(x, margin = c("documents", "features"), doclabels = NULL, 
+textplot_scale1d <- function(x, 
+                             margin = c("documents", "features"), 
+                             doclabels = NULL, 
                              sort = TRUE, groups = NULL, 
                              highlighted = NULL, alpha = 0.7, 
                              highlighted_color = "black") {
     UseMethod("textplot_scale1d")
 }
 
-#' @noRd
+#' @export
+textplot_scale1d.default <-  function(x,
+                                      margin = c("documents", "features"), 
+                                      doclabels = NULL, 
+                                      sort = TRUE, groups = NULL, 
+                                      highlighted = NULL, alpha = 0.7, 
+                                      highlighted_color = "black") {
+    stop(friendly_class_undefined_message(class(x), "textplot_scale1d"))
+}
+
 #' @importFrom stats reorder aggregate
 #' @importFrom ggplot2 ggplot aes geom_point element_blank geom_pointrange 
 #' @importFrom ggplot2 coord_flip xlab ylab theme_bw geom_text theme geom_point
 #' @importFrom ggplot2 facet_grid element_line
 #' @export
-textplot_scale1d.textmodel_wordfish_fitted <-  function(x, 
-                                                        margin = c("documents", "features"), 
-                                                        doclabels = NULL, 
-                                                        sort = TRUE, groups = NULL, 
-                                                        highlighted = NULL, alpha = 0.7, 
-                                                        highlighted_color = "black") {
+textplot_scale1d.textmodel_wordfish <-  function(x, 
+                                                 margin = c("documents", "features"), 
+                                                 doclabels = NULL, 
+                                                 sort = TRUE, 
+                                                 groups = NULL, 
+                                                 highlighted = NULL, 
+                                                 alpha = 0.7, 
+                                                 highlighted_color = "black") {
     margin <- match.arg(margin)
-    if (is.null(doclabels)) doclabels <- x@docs
+    if (is.null(doclabels)) doclabels <- x$docs
     
     if (margin == "documents") {
-        p <- textplot_scale1d_documents(coef(x)$coef_document, coef(x)$coef_document_se, 
-                                        doclabels = doclabels, sort = sort, groups = groups) +
+        p <- textplot_scale1d_documents(x$theta,
+                                        x$se.theta, 
+                                        doclabels = doclabels, 
+                                        sort = sort, 
+                                        groups = groups) +
             ylab("Estimated theta")
     } else if (margin == "features") {
-        p <- textplot_scale1d_features(coef(x)$coef_feature, 
-                                       weight = coef(x)$coef_feature_offset, 
-                                       featlabels = x@features,
+        p <- textplot_scale1d_features(x$beta, 
+                                       weight = x$psi, 
+                                       featlabels = x$features,
                                        highlighted = highlighted, alpha = alpha,
                                        highlighted_color = highlighted_color) +
             xlab("Estimated beta") +
@@ -107,59 +120,85 @@ textplot_scale1d.textmodel_wordfish_fitted <-  function(x,
     } 
     apply_theme(p)
 } 
-
      
-#' @noRd
 #' @importFrom stats reorder aggregate
 #' @importFrom ggplot2 ggplot aes geom_point element_blank geom_pointrange 
 #' @importFrom ggplot2 coord_flip xlab ylab theme_bw geom_text theme geom_point
 #' @importFrom ggplot2 facet_grid element_line
+#' @method textplot_scale1d predict.textmodel_wordscores
 #' @export
-textplot_scale1d.textmodel_wordscores_predicted <- function(x, 
-                                                            margin = c("documents", "features"), 
-                                                            doclabels = NULL, 
-                                                            sort = TRUE, groups = NULL, 
-                                                            highlighted = NULL, alpha = 0.7, 
-                                                            highlighted_color = "black") {
+textplot_scale1d.predict.textmodel_wordscores <- function(x, 
+                                                          margin = c("documents", "features"), 
+                                                          doclabels = NULL, 
+                                                          sort = TRUE, 
+                                                          groups = NULL, 
+                                                          highlighted = NULL, 
+                                                          alpha = 0.7, 
+                                                          highlighted_color = "black") {
     margin <- match.arg(margin)
-    if (is.null(doclabels)) doclabels <- docnames(x@newdata)
+    if (is.null(doclabels)) doclabels <- get_docnames(x)
     
     if (margin == "documents") {
-        p <- textplot_scale1d_documents(coef(x)$coef_document, coef(x)$coef_document_se, 
-                                        doclabels = doclabels, sort = sort, groups = groups) +
-            
+        p <- textplot_scale1d_documents(get_fitted(x), 
+                                        get_sefit(x), 
+                                        doclabels = doclabels, 
+                                        sort = sort, 
+                                        groups = groups) +
               ylab("Document position")
         
     } else if (margin == "features") {
-        p <- textplot_scale1d_features(x@Sw, 
-                                       weight = log(colSums(x@x[, names(x@Sw)])),
-                                       featlabels = featnames(x@x[, names(x@Sw)]),
-                                       highlighted = highlighted, alpha = alpha,
-                                       highlighted_color = highlighted_color) +
-            xlab("Word score") +
-            ylab("log(term frequency)") 
+        stop("This margin can only be run on a fitted wordscores object.")
     } 
     apply_theme(p)
 }
 
-#' @noRd
+
+#' @export
+textplot_scale1d.textmodel_wordscores <- function(x, 
+                                                  margin = c("features", "documents"), 
+                                                  doclabels = NULL, 
+                                                  sort = TRUE, 
+                                                  groups = NULL, 
+                                                  highlighted = NULL, 
+                                                  alpha = 0.7, 
+                                                  highlighted_color = "black") {
+    margin <- match.arg(margin)
+    if (margin == "documents") {
+        stop("This margin can only be run on a predicted wordscores object.")
+    } else if (margin == "features") {
+        p <- textplot_scale1d_features(x$wordscores, 
+                                       weight = log(colSums(x$x[, names(x$wordscores)])),
+                                       featlabels = names(x$wordscores),
+                                       highlighted = highlighted, alpha = alpha,
+                                       highlighted_color = highlighted_color) +
+            xlab("Word score") +
+            ylab("log(term frequency)") 
+        apply_theme(p)
+    }
+}
+
 #' @importFrom stats reorder aggregate
 #' @importFrom ggplot2 ggplot aes geom_point element_blank geom_pointrange 
 #' @importFrom ggplot2 coord_flip xlab ylab theme_bw geom_text theme geom_point
 #' @importFrom ggplot2 facet_grid element_line
 #' @export
-textplot_scale1d.textmodel_ca_fitted <- function(x, 
-                                                    margin = c("documents", "features"), 
-                                                    doclabels = NULL, 
-                                                    sort = TRUE, groups = NULL, 
-                                                    highlighted = NULL, alpha = 0.7, 
-                                                    highlighted_color = "black") {
+textplot_scale1d.textmodel_ca <- function(x, 
+                                          margin = c("documents", "features"), 
+                                          doclabels = NULL, 
+                                          sort = TRUE, 
+                                          groups = NULL, 
+                                          highlighted = NULL, 
+                                          alpha = 0.7, 
+                                          highlighted_color = "black") {
     margin <- match.arg(margin)
     if (is.null(doclabels)) doclabels <- x$rownames
     
     if (margin == "documents") {
-        p <- textplot_scale1d_documents(coef(x)$coef_document, coef(x)$coef_document_se, 
-                                        doclabels = doclabels, sort = sort, groups = groups) +
+        p <- textplot_scale1d_documents(coef(x)$coef_document, 
+                                        coef(x)$coef_document_se, 
+                                        doclabels = doclabels, 
+                                        sort = sort, 
+                                        groups = groups) +
             ylab("Document position")
         
     } else {
@@ -171,7 +210,8 @@ textplot_scale1d.textmodel_ca_fitted <- function(x,
 
 # internal fns --------
 
-textplot_scale1d_documents <- function(x, se, doclabels, sort = TRUE, groups = NULL) {
+textplot_scale1d_documents <- function(x, se, doclabels, sort = TRUE, 
+                                       groups = NULL) {
 
     if (!is.null(doclabels))
         stopifnot(length(doclabels) == length(x))
@@ -180,7 +220,8 @@ textplot_scale1d_documents <- function(x, se, doclabels, sort = TRUE, groups = N
     
     if (sort & !is.null(groups)) {
         temp_medians <- aggregate(x, list(groups), median, na.rm = TRUE)
-        groups <- factor(groups, levels = temp_medians[order(temp_medians$x, decreasing = TRUE), 1])
+        groups <- factor(groups, 
+                         levels = temp_medians[order(temp_medians$x, decreasing = TRUE), 1])
     }
     
     theta <- lower <- upper <- NULL
@@ -200,7 +241,8 @@ textplot_scale1d_documents <- function(x, se, doclabels, sort = TRUE, groups = N
     p <- p + 
         coord_flip() + 
         geom_point(size = 1) +
-        geom_pointrange(aes(ymin = lower, ymax = upper), lwd = .25, fatten = .4) +
+        geom_pointrange(aes(ymin = lower, ymax = upper), 
+                        lwd = .25, fatten = .4) +
         xlab(NULL)
     if (!is.null(groups)) {
         p <- p + facet_grid(as.factor(groups) ~ ., scales = "free_y", space = "free")
@@ -226,7 +268,8 @@ textplot_scale1d_features <- function(x, weight, featlabels,
                   color = highlighted_color) +
         xlab("Beta") +
         ylab("Psi") + 
-        theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank())
+        theme(panel.grid.major = element_blank(), 
+              panel.grid.minor = element_blank())
     p
 }
 
@@ -245,3 +288,38 @@ apply_theme <- function(p) {
               # panel.spacing = grid::unit(0.1, "lines"),
               panel.grid.major.y = element_line(linetype = "dotted"))
 }
+
+
+# internal functions --------
+
+get_docnames <- function(x) {
+    if (is.null(dim(x))) {
+        dnames <- names(x)
+    } else {
+        if (is.list(x)) 
+            dnames <- row.names(x$fit)
+        else
+            dnames <- row.names(x)
+    }
+    dnames
+}
+
+get_fitted <- function(x) {
+    if (is.list(x)) x <- x$fit
+    if (is.numeric(x)) {
+        fit <- x
+    } else {
+        fit <- x[, "fit"]
+    }
+    fit
+}
+
+get_sefit <- function(x) {
+    # se is zero if prediction is estimates
+    if (is.numeric(x)) return(rep(0, length(x)))
+    if (is.list(x) && !is.null(x$se.fit)) {
+        return(x$se.fit)
+    } else {
+        return((x[, "fit"] - x[, "lwr"]) / 1.96)  
+    }
+}
