@@ -33,12 +33,12 @@
 #'   \code{"self-storage"} becomes \code{c("self", "storage")}.  Default is 
 #'   \code{FALSE} to preserve such words as is, with the hyphens.  Only applies 
 #'   if \code{what = "word"}.
-#' @param remove_separators remove Separators and separator characters (spaces 
+#' @param remove_separators remove separators and separator characters (spaces 
 #'   and variations of spaces, plus tab, newlines, and anything else in the 
 #'   Unicode "separator" category) when \code{remove_punct=FALSE}.  Only 
 #'   applicable for \code{what = "character"} (when you probably want it to be 
 #'   \code{FALSE}) and for \code{what = "word"} (when you probably want it to be
-#'   \code{TRUE}).  Note that if \code{what = "word"} and you set 
+#'   \code{TRUE}).  Note that if \code{what = "word"} and  
 #'   \code{remove_punct = TRUE}, then \code{remove_separators} has no effect.  Use
 #'   carefully.
 #' @param ngrams integer vector of the \emph{n} for \emph{n}-grams, defaulting 
@@ -383,27 +383,25 @@ is.tokens <- function(x) "tokens" %in% class(x)
 #' @return a list the serialized tokens found in each text
 #' @importFrom fastmatch fmatch
 #' @keywords internal tokens
-tokens_serialize <- function(x, types_reserved, ...) {
+tokens_serialize <- function(x, types_reserved = NULL, ...) {
     
     attrs <- attributes(x)
     types <- unique(unlist(x, use.names = FALSE))
+    types <- types[types != '']  # remove empty tokens
     
-    # special handling for "" pad
-    if (any(types[!is.na(types)] == "")) {
-        types <- c("", setdiff(types, ""))
-    }
-
-    if (!missing(types_reserved)) {
+    if (!is.null(types_reserved)) {
         types <- c(types_reserved, setdiff(types, types_reserved))
     }
     x <- lapply(x, function(x) {
         id <- fastmatch::fmatch(x, types)
-        if (length(types) && !is.na(types[1]) && types[1] == "") id <- id - 1
-        id[!is.na(id)]
+        is_na <- is.na(id)
+        if (length(is_na) > 0) {
+            id[!is_na]
+        } else {
+            integer()
+        }
     })
     
-    types <- types[types != '']  # remove empty tokens
-    
     attributes(x) <- attrs
     attr(x, "types") <- stri_trans_nfc(types) # unicode normalization
     class(x) <- "tokens"
@@ -582,7 +580,7 @@ tokens_internal <- function(x, what = c("word", "sentence", "character", "fastes
         } else if (what == "fastestword") {
             temp <- tokens_word(x[[i]], what, FALSE, FALSE, FALSE, FALSE, verbose)
         } else if (what == "character") {
-            temp <- tokens_character(x[[i]],remove_punct, remove_symbols, remove_separators, verbose)
+            temp <- tokens_character(x[[i]], remove_punct, remove_symbols, remove_separators, verbose)
         } else if (what == "sentence") {
             temp <- tokens_sentence(x[[i]], verbose)
         } else {
@@ -661,7 +659,7 @@ tokens_word <- function(txt,
     if (what=="fastestword") {
         tok <- stri_split_fixed(txt, " ")
     } else if (what=="fasterword") {
-        tok <- stri_split_charclass(txt, "\\p{WHITE_SPACE}")
+        tok <- stri_split_regex(txt, "\\p{Z}+")
     } else {
         txt <- stri_replace_all_regex(txt, "[\uFE00-\uFE0F]", '') # remove variant selector
         txt <- stri_replace_all_regex(txt, "\\s[\u0300-\u036F]", '') # remove whitespace with diacritical marks
