@@ -189,3 +189,26 @@ friendly_class_undefined_message <- function(object_class, function_name) {
          " objects.")
 }
 
+#' Check if font is available on the system
+#' 
+#' This function checks if custom font is available to \pkg{ggplot} and
+#' \pkg{graphics} APIs.
+#' @param font name of a font to be checked if available on the system.
+#' @return character string
+#' @keywords internal
+check_font <- function(font) {
+    
+    if (is.null(font)) {
+        font <- ""
+    } else {
+        msg <- paste0(font, ' is not found on your system.')
+        if (.Platform$OS.type == 'windows') {
+            if (!font %in% names(grDevices::windowsFonts()))
+                stop(msg, ' Run extrafont::import_font() and extrafont::loadfonts(device = "win") to use custom fonts.')
+        } else {
+            if (!font %in% c('sans', 'serif', 'mono', extrafont::fonts()))
+                stop(msg, ' Run extrafont::import_font() to use custom fonts.')
+        }
+    }
+    return(font)
+}
