@@ -18,13 +18,13 @@ test_that("textmodel-ca (rsvd) works as expected as ca::ca", {
     expect_equal(wca$rowinertia, wtca$rowinertia, tolerance = 1e-6)
     expect_equal(wca$colinertia, wtca$colinertia, tolerance = 1e-6)
     
-    expect_equal(wca$sv[1:length(wtca$sv)], wtca$sv, tolerance = 1e-6)
+    expect_equal(wca$sv[seq_along(wtca$sv)], wtca$sv, tolerance = 1e-6)
 })
 
 test_that("textmodel-ca works as expected as ca::ca : use mt", {
     skip_if_not_installed("ca")
     wca <- ca::ca(as.matrix(ie2010dfm))
-    wtca <- textmodel_ca(ie2010dfm, threads = 2, sparse = TRUE)
+    wtca <- textmodel_ca(ie2010dfm, sparse = TRUE)
     
     expect_gt(cor(wca$rowdist, wtca$rowdist), 0.99)
     expect_gt(cor(wca$coldist, wtca$coldist), 0.99)
@@ -38,7 +38,7 @@ test_that("textmodel-ca works as expected as ca::ca : use mt", {
     expect_gt(cor(wca$rowinertia, wtca$rowinertia), 0.99)
     expect_gt(cor(wca$colinertia, wtca$colinertia), 0.99)
     
-    cc <- cor(wca$sv[1:length(wtca$sv)], wtca$sv)
+    cc <- cor(wca$sv[seq_along(wtca$sv)], wtca$sv)
     expect_gt(cc, 0.99)
 })
 
@@ -58,14 +58,14 @@ test_that("textmodel-ca works as expected as ca::ca: for given number of dimensi
     expect_equal(wca$rowinertia, wtca$rowinertia, tolerance = 1e-6)
     expect_equal(wca$colinertia, wtca$colinertia, tolerance = 1e-6)
     
-    expect_equal(wca$sv[1:length(wtca$sv)], wtca$sv, tolerance = 1e-6)
+    expect_equal(wca$sv[seq_along(wtca$sv)], wtca$sv, tolerance = 1e-6)
 })
 
 test_that("textmodel-ca(sparse) works as expected on another dataset", {
     usdfm <- dfm(data_corpus_inaugural, verbose = FALSE)
     skip_if_not_installed("ca")
     wca <- ca::ca(as.matrix(usdfm))
-    wtca <- textmodel_ca(usdfm, threads = 2, sparse = TRUE)
+    wtca <- textmodel_ca(usdfm, sparse = TRUE)
     
     expect_gt(cor(wca$rowdist, wtca$rowdist), 0.99)
     expect_gt(cor(wca$coldist, wtca$coldist), 0.99)
@@ -79,23 +79,10 @@ test_that("textmodel-ca(sparse) works as expected on another dataset", {
     expect_gt(cor(wca$rowinertia, wtca$rowinertia), 0.99)
     expect_gt(cor(wca$colinertia, wtca$colinertia), 0.99)
     
-    cc <- cor(wca$sv[1:length(wtca$sv)], wtca$sv)
+    cc <- cor(wca$sv[seq_along(wtca$sv)], wtca$sv)
     expect_gt(cc, 0.99)
 })
-# test_that("textmodel-ca generates a ca::ca: object, which can call other functions from ca package", {
-#     skip_if_not_installed("ca")
-#     wca <- ca::ca(smoke)
-#     wtca <- textmodel_ca(as.dfm(smoke), nd=5)
-#     expect_equal(wca$rowmass, wtca$rowmass, tolerance = 1e-6)
-#     
-#     # plot method
-#     #p <- ca::plot.ca(wtca)
-#     #expect_equal(dimnames(p$rows)[[2]][1], "Dim1")
-#     
-#     # summary method
-#     s <- summary(wtca)
-#     expect_equal(attr(s, "class"), "summary.ca")
-# })
+
 
 test_that("ca coefficients methods work", {
     camodel <- textmodel_ca(data_dfm_lbgexample)
@@ -115,10 +102,3 @@ test_that("ca textplot_scale1d method works", {
     )
 })
 
-test_that("ca sparse = FALSE with threads > 1 issues warning (#663)", {
-    expect_warning(
-        textmodel_ca(data_dfm_lbgexample, sparse = FALSE, threads = 2),
-        "threads reset to 1 when sparse = FALSE"
-    )
-})
-
